@@ -1422,32 +1422,34 @@ gchar *gfile_display_name(GFile *uri, GFileInfo *finfo) {
 }
 
 gboolean gfile_uri_is_parent(GFile *parent, GFile *child, gboolean recursive) {
+	gboolean retval = FALSE;
+	GFile *tmp, *tmp2;
+
 	if (parent == NULL) {
 		DEBUG_MSG("gfile_uri_is_parent, parent=NULL ??\n");
 		return FALSE;
-	} else if (child == NULL) {
+	} 
+	if (child == NULL) {
 		DEBUG_MSG("gfile_uri_is_parent, child=NULL ??\n");
 		return FALSE;
-	} else {
-		gboolean retval = FALSE;
-		GFile *tmp, *tmp2=child;
-		
-		g_object_ref(child);
-		do {
-			tmp = g_file_get_parent(tmp2);
-			g_object_unref(tmp2);
-			if (tmp == NULL) {
-				break;
-			}
-			retval = g_file_equal(tmp,parent);
-			tmp2 = tmp;
-		} while (recursive == TRUE && retval != TRUE);
-		if (tmp) {
-			g_object_unref(tmp);
-		}
-		return retval;
 	}
+	g_object_ref(child);
+	tmp2=child;
+	do {
+		tmp = g_file_get_parent(tmp2);
+		g_object_unref(tmp2);
+		if (tmp == NULL) {
+			break;
+		}
+		retval = g_file_equal(tmp,parent);
+		tmp2 = tmp;
+	} while (recursive == TRUE && retval != TRUE);
+	if (tmp) {
+		g_object_unref(tmp);
+	}
+	return retval;
 }
+
 gchar *get_hostname_from_uri(GFile *uri) {
 	gchar *tmp, *begin, *end1, *end2, *end, *retval;
 	
