@@ -31,12 +31,37 @@ void match_add_autocomp_item(Tscantable * st, guint16 matchnum, const gchar * au
 							 const gchar * autocomplete_append, guint8 autocomplete_backup_cursor);
 void match_set_reference(Tscantable * st, guint16 matchnum, const gchar * reference);
 void compile_existing_match(Tscantable * st, guint16 matchnum, gint16 context);
-guint16 add_keyword_to_scanning_table(Tscantable * st, gchar * pattern, const gchar * lang,
+
+void
+pattern_set_blockmatch(Tscantable * st, guint16 matchnum, 
+							gboolean starts_block,
+							gboolean ends_block, 
+							guint blockstartpattern,
+							const gchar * blockhighlight);
+
+void
+pattern_set_runtime_properties(Tscantable * st, guint16 matchnum, 
+								const gchar * selfhighlight,
+								gint16 nextcontext,
+								gboolean tagclose_from_blockstack,
+								gboolean stretch_blockstart,
+								guint8 identmode,
+								gboolean identjump,
+								gboolean identautocomp);
+
+guint16
+add_pattern_to_scanning_table(Tscantable * st, gchar * pattern,
+								gboolean is_regex,
+								gboolean case_insens,
+								gint16 context);
+
+
+/*guint16 add_keyword_to_scanning_table(Tscantable * st, gchar * pattern, const gchar * lang,
 							  const gchar * selfhighlight, const gchar * blockhighlight, gboolean is_regex,
 							  gboolean case_insens, gint16 context, gint16 nextcontext, gboolean starts_block,
 							  gboolean ends_block, guint blockstartpattern, 
 							  gboolean tagclose_from_blockstack, gboolean stretch_blockstart,
-							  guint8 identmode, gboolean identjump, gboolean identautocomp);
+							  guint8 identmode, gboolean identjump, gboolean identautocomp);*/
 void print_DFA(Tscantable * st, char start, char end);
 Tscantable *scantable_new(guint size_table, guint size_matches, guint size_contexts);
 Tscantable *bftextview2_scantable_new(GtkTextBuffer * buffer);
