@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * bfwin_uimanager.c
  *
- * Copyright (C) 2011 James Hayward and Olivier Sessink
+ * Copyright (C) 2011, 2012 James Hayward and Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 /*#define DEBUG*/
 
 #include <stdlib.h>				/* atoi */
@@ -633,6 +632,18 @@ ui_select_block(GtkAction * action, gpointer user_data)
 	select_between_matching_block_boundaries(BFWIN(user_data)->current_document);
 }
 
+static void
+ui_duplicate_line(GtkAction * action, gpointer user_data)
+{
+	duplicate_line(BFWIN(user_data)->current_document);
+}
+
+static void
+ui_delete_line(GtkAction * action, gpointer user_data)
+{
+	delete_line(BFWIN(user_data)->current_document);
+}
+
 static void
 ui_rewrap_lines(GtkAction * action, gpointer user_data)
 {
@@ -787,10 +798,10 @@ static const GtkActionEntry global_actions[] = {
 	{"DocTabSizeReset", NULL, N_("_Reset"), NULL, N_("Reset document tab size"),
 	 G_CALLBACK(ui_tab_size_reset)},
 #ifdef HAVE_LIBENCHANT
-	{"DocumentRescan", NULL, N_("Rescan Syntax & S_pelling"), "F5", N_("Rescan document highlighting"),
+	{"DocumentRescan", NULL, N_("Rescan Synta_x & Spelling"), "F5", N_("Rescan document highlighting"),
 	 G_CALLBACK(ui_highlighting_update)},
 #else
-	{"DocumentRescan", NULL, N_("_Rescan Syntax"), "F5", N_("Rescan document highlighting"),
+	{"DocumentRescan", NULL, N_("Rescan Synta_x"), "F5", N_("Rescan document highlighting"),
 	 G_CALLBACK(ui_highlighting_update)},
 #endif
 	{"ProjectNew", NULL, N_("_New Project..."), NULL, N_("New project"), G_CALLBACK(ui_project_new)},
@@ -801,10 +812,14 @@ static const GtkActionEntry global_actions[] = {
 	 G_CALLBACK(ui_toggle_comment)},
 	 {"SelectBlock", NULL, N_("Select _Block"), "<shift><control>b", N_("Select block, use multiple times to select parent blocks "),
 	 G_CALLBACK(ui_select_block)},
+	 {"DuplicateLine", NULL, N_("Duplica_te Line"), "<control>d", N_("Duplicate the current line"),
+	 G_CALLBACK(ui_duplicate_line)},
+	 {"DeleteLine", NULL, N_("Delete Line"), "<control>y", N_("Delete the current line"),
+	 G_CALLBACK(ui_delete_line)},
 	{"WordCount", NULL, N_("Word Cou_nt"), NULL, N_("Word count"), G_CALLBACK(ui_word_count)},
-	{"IndentingToSpaces", NULL, N_("Indenting To S_paces"), NULL, N_("Indenting to spaces"),
+	{"IndentingToSpaces", NULL, N_("Indenting to S_paces"), NULL, N_("Indenting to spaces"),
 	 G_CALLBACK(ui_indenting_to_spaces)},
-	{"IndentingToTabs", NULL, N_("Indenting To T_abs"), NULL, N_("Indenting to tabs"),
+	{"IndentingToTabs", NULL, N_("Indenting to T_abs"), NULL, N_("Indenting to tabs"),
 	 G_CALLBACK(ui_indenting_to_tabs)},
 	{"JoinLines", NULL, N_("_Join Lines Together"), NULL, N_("Join lines together"),
 	 G_CALLBACK(ui_join_lines)},
@@ -815,7 +830,7 @@ static const GtkActionEntry global_actions[] = {
 	{"RewrapLines", NULL, N_("Rewrap _Lines"), NULL, N_("Rewrap lines"), G_CALLBACK(ui_rewrap_lines)},
 	{"StripTrailingWhitespace", NULL, N_("Strip T_railing Whitespace"), NULL, N_("Strip trailing whitespace"),
 	 G_CALLBACK(ui_strip_trailing_whitespace)},
-	 {"BrowserPreview", BF_STOCK_BROWSER_PREVIEW, /*_("Preview in browser")*/NULL, NULL, /*N_("Preview in browser")*/NULL,
+	 {"BrowserPreview", BF_STOCK_BROWSER_PREVIEW, N_("Preview in browser"), NULL, N_("Preview in browser"),
 	 G_CALLBACK(ui_browser_preview)}
 };
 
@@ -857,7 +872,7 @@ static const GtkActionEntry document_actions[] = {
 	 G_CALLBACK(ui_file_save)},
 	{"FileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _as..."), "<shift><control>S", N_("Save file as"),
 	 G_CALLBACK(ui_file_save_as)},
-	{"FileSaveAll", NULL, N_("Sav_e All"), NULL, N_("Save all files"), G_CALLBACK(ui_file_save_all)},
+	{"FileSaveAll", NULL, N_("Sav_e All"), "<alt><shift>s", N_("Save all files"), G_CALLBACK(ui_file_save_all)},
 	{"FileRevert", GTK_STOCK_REVERT_TO_SAVED, N_("Rever_t to Saved"), NULL, N_("Reload current file"),
 	 G_CALLBACK(ui_file_reload)},
 	{"FileRename", NULL, N_("Rena_me..."), "F2", N_("Rename file"), G_CALLBACK(ui_file_rename)},
@@ -898,11 +913,11 @@ static const GtkActionEntry edit_actions[] = {
 };
 
 static const GtkActionEntry find_replace_actions[] = {
-	{"Find", GTK_STOCK_FIND, N_("_Find..."), "<control>F", N_("Find"), G_CALLBACK(ui_find)},
+	{"Find", GTK_STOCK_FIND, N_("_Find"), "<control>F", N_("Show find bar"), G_CALLBACK(ui_find)},
 	{"FindAgain", NULL, N_("Find A_gain"), "<control>G", N_("Find again"), G_CALLBACK(ui_find_again)},
-	{"FindSelection", NULL, N_("Find from Clip_board"), "<shift><control>F", N_("Find from clipboard"),
+	{"FindSelection", NULL, N_("Find from Clip_board"), "<shift><control>F", N_("Find from clipboard or selection"),
 	 G_CALLBACK(ui_find_from_clipboard)},
-	{"Replace", GTK_STOCK_FIND_AND_REPLACE, N_("Re_place..."), "<control>H", N_("Replace"),
+	{"Replace", GTK_STOCK_FIND_AND_REPLACE, N_("Advanced Find & Re_place..."), "<control>H", N_("Advanced Find and Replace"),
 	 G_CALLBACK(ui_replace)}/*,
 	{"ReplaceAgain", NULL, N_("Replace Agai_n"), "<shift><control>H", N_("Replace again"),
 	 G_CALLBACK(ui_replace_again)}*/
@@ -957,6 +972,9 @@ lang_mode_menu_create(Tbfwin * bfwin)
 	GList *list, *freelist;
 	gint value = 0;
 
+	if (!bfwin->uimanager)
+		return;
+
 	if (!bfwin->lang_mode_group) {
 		bfwin->lang_mode_group = gtk_action_group_new("LangModeActions");
 		gtk_ui_manager_insert_action_group(bfwin->uimanager, bfwin->lang_mode_group, 1);
@@ -1234,6 +1252,13 @@ dynamic_menu_item_create(GtkUIManager *uimanager, GtkActionGroup *action_group,
 						GCallback callback, gpointer callbackdata, gpointer actiondata)
 {
 	GtkAction *action;
+	/* avoid duplicate actions */
+	action = gtk_action_group_get_action(action_group, action_name);
+	if (action) {
+		g_warning("duplicate action %s, ignoring\n", action_name);
+		return;
+	}
+	
 	if (radio_value == -1) {
 		action = gtk_action_new(action_name, action_label, NULL, NULL);
 	} else {
@@ -1246,9 +1271,11 @@ dynamic_menu_item_create(GtkUIManager *uimanager, GtkActionGroup *action_group,
 		*radio_group = gtk_radio_action_get_group(GTK_RADIO_ACTION(action));
 	}
 
-	gtk_action_group_add_action(action_group, action);
+/*	gtk_action_group_add_action(action_group, action);*/
+	gtk_action_group_add_action_with_accel(action_group, action, "");
 	g_object_set_data(G_OBJECT(action), "adata", actiondata);
 	g_signal_connect(G_OBJECT(action), "activate", callback, callbackdata);
+	DEBUG_MSG("dynamic_menu_item_create, add action %s at path %s\n", action_name, path);
 	gtk_ui_manager_add_ui(uimanager, merge_id,path, action_name,action_name, GTK_UI_MANAGER_MENUITEM, TRUE);
 	g_object_unref(action);
 } 
@@ -1645,18 +1672,19 @@ static void recent_create_backend(Tbfwin *bfwin, const gchar *menupath, GList *r
 		}
 	}
 	g_list_free(list);
-
 	for (tmplist = g_list_first(recentlist); tmplist; tmplist = tmplist->next) {
 		if (num > main_v->props.max_recent_files)
 			break;
 		recent_menu_add(bfwin, GTK_MENU(menu), (const gchar *)tmplist->data);
 		num++;
 	}
+	gtk_widget_show_all(menuitem);
 }
 
 void
 bfwin_recent_menu_create(Tbfwin *bfwin, gboolean only_update_session)
 {
+	DEBUG_MSG("bfwin_recent_menu_create, only_update_session=%d\n",only_update_session);
 	recent_create_backend(bfwin, "/MainMenu/FileMenu/FileOpenRecent", bfwin->session->recent_files);
 	if (!only_update_session)
 		recent_create_backend(bfwin, "/MainMenu/ProjectMenu/ProjectOpenRecent", main_v->globses.recent_projects);
