@@ -28,7 +28,7 @@
 #ifndef __BLUEFISH_H_
 #define __BLUEFISH_H_
 
-/*#define MEMORY_LEAK_DEBUG*/
+#define MEMORY_LEAK_DEBUG
 /*#define DEBUG_PATHS*/
 
 #define ENABLEPLUGINS
@@ -166,6 +166,8 @@ scanning in a lower priority timeout than the language file notice
 so a newly loaded language file uses a priority 113 event to notice all documents to be rescanned. */
 #define BUILD_LANG_FINISHED_PRIORITY 113
 
+#define BFLANGSCAN_FINISHED_PRIORITY 101
+
 /*********************/
 /* undo/redo structs */
 /*********************/
@@ -276,6 +278,7 @@ typedef struct {
 	gint leave_to_window_manager;	/* don't set any dimensions, leave all to window manager */
 	gint restore_dimensions;	/* use the dimensions as used the previous run */
 	gint left_panel_left;		/* 1 = left, 0 = right */
+	gint save_accelmap; 	/* save the accelerator map on exit */
 	gint max_recent_files;		/* length of Open Recent list */
 	gint max_dir_history;		/* length of directory history */
 	gint backup_file;			/* wheather to use a backup file */
@@ -390,6 +393,7 @@ typedef struct {
 	gint snr3_casesens;
 	gint snr3_escape_chars;
 	gint snr3_dotmatchall;
+	gint snr3_recursion_level;
 
 	gint sync_delete_deprecated;
 	gint sync_include_hidden;
@@ -412,7 +416,7 @@ typedef struct {
 	gint outputb_show_all_output;
 	gint convertcolumn_horizontally;
 	gint display_right_margin;
-	/* 38 * sizeof(gint) */
+	/* 39 * sizeof(gint) */
 	/* IF YOU EDIT THIS STRUCTURE PLEASE EDIT THE CODE IN PROJECT.C THAT COPIES
 	   A Tsessionvar INTO A NEW Tsessionvar AND ADJUST THE SIZES!!!!!!!!!!!!!!!!!!!!!! */
 #ifdef HAVE_LIBENCHANT
@@ -618,6 +622,8 @@ void bluefish_exit_request(void);
 #define GDK_KEY_7 GDK_7
 #define GDK_KEY_8 GDK_8
 #define GDK_KEY_9 GDK_9
+#define GDK_KEY_F1 GDK_F1 
+#define GDK_KEY_F12 GDK_F12
 #define GDK_KEY_Delete GDK_Delete
 #define GDK_KEY_BackSpace GDK_BackSpace
 #define GDK_KEY_KP_Delete GDK_KP_Delete
