@@ -21,7 +21,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "bluefish.h"
-
+#include "pixmap.h"
 #include "pixmaps/toolbar_icons.c"
 
 typedef struct {
@@ -122,6 +122,34 @@ GtkWidget *new_pixmap(gint type) {
 	return widget;
 }
 
+void
+register_bf_stock_icons(void)
+{
+	static struct {
+		const guint8 *data;
+		gchar *stock_id;
+	} stock_icons[] = {
+			{pixmap_view_in_browser, BF_STOCK_BROWSER_PREVIEW}
+	};
+
+	GtkIconFactory *icon_factory;
+	GtkIconSet *icon_set;
+	gint i;
+
+	icon_factory = gtk_icon_factory_new();
+	for (i = 0; i < G_N_ELEMENTS(stock_icons); i++) {
+		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline(-1,stock_icons[i].data,FALSE,NULL);
+		icon_set = gtk_icon_set_new_from_pixbuf(pixbuf);
+		g_object_unref(pixbuf);
+		gtk_icon_factory_add(icon_factory, stock_icons[i].stock_id, icon_set);
+		gtk_icon_set_unref(icon_set);
+	}
+	gtk_icon_factory_add_default(icon_factory);
+	g_object_unref(icon_factory);
+}
+
+
+
 void set_default_icon(void) {
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline(-1,pixmap_bluefish_icon1,FALSE,NULL);
 	GList *tmplist = g_list_append(NULL, pixbuf);
