@@ -30,22 +30,25 @@
 #ifdef ENABLE_NLS
 #undef _
 #define _(String) dgettext(PACKAGE"_plugin_htmlbar", String)
-#endif    
+#endif
 
 typedef struct {
-	GHashTable* lookup;
+	GHashTable *lookup;
 	GList *quickbar_items;
 } Thtmlbar;
 
 typedef struct {
 	Tbfwin *bfwin;
 	GtkWidget *handlebox;
+	GtkActionGroup *actiongroup;
+	GtkWidget *quickbar_toolbar;
 } Thtmlbarwin;
 
 typedef struct {
 	gint view_htmlbar;
+	gint notebooktab;
 } Thtmlbarsession;
 
 extern Thtmlbar htmlbar_v;
 
-#endif /* __HTMLBAR_H_ */
+#endif							/* __HTMLBAR_H_ */
