@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * htmlbar_uimanager.c
  *
- * Copyright (C) 2005-2011 Olivier Sessink
+ * Copyright (C) 2005-2012 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include "../bluefish.h"
 #include "../bfwin_uimanager.h"
 #include "../document.h"
+#include "../stringlist.h"
 #include "cap.h"
 #include "html.h"
 #include "html2.h"
@@ -74,6 +75,12 @@ htmlbar_dialog_body(GtkAction * action, gpointer user_data)
 	body_dialog(BFWIN(user_data), NULL);
 }
 
+static void
+htmlbar_dialog_script(GtkAction * action, gpointer user_data)
+{
+	script_dialog(BFWIN(user_data), NULL);
+}
+
 static void
 htmlbar_dialog_color(GtkAction * action, gpointer user_data)
 {
@@ -231,7 +238,7 @@ htmlbar_dialog_span(GtkAction * action, gpointer user_data)
 }
 
 static void
-htmlbar_dialog_style(GtkAction * action, gpointer user_data)
+htmlbar_misc_style(GtkAction * action, gpointer user_data)
 {
 	gchar *tmp = g_strdup_printf("<%s=\"text/css\"><!--\n", cap("STYLE TYPE"));
 	doc_insert_two_strings(BFWIN(user_data)->current_document, tmp, cap("\n--></STYLE>"));
@@ -334,7 +341,7 @@ htmlbar_insert_align_right_tag(GtkAction * action, gpointer user_data)
 static void
 htmlbar_insert_anchor_tag(GtkAction * action, gpointer user_data)
 {
-	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<A>"), cap("</A>"));
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<A HREF=\""), cap("\"></A>"));
 }
 
 static void
@@ -453,9 +460,21 @@ htmlbar_insert_form_tag(GtkAction * action, gpointer user_data)
 }
 
 static void
-htmlbar_insert_form_button_tag(GtkAction * action, gpointer user_data)
+htmlbar_insert_form_fieldset_tag(GtkAction * action, gpointer user_data)
 {
-	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<BUTTON>"), cap("</BUTTON>"));
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<FIELDSET>"), cap("</FIELDSET>"));
+}
+
+static void
+htmlbar_insert_form_legend_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<LEGEND>"), cap("<LEGEND/>"));
+}
+
+static void
+htmlbar_insert_form_label_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<LABEL>"), cap("</LABEL>"));
 }
 
 static void
@@ -465,12 +484,67 @@ htmlbar_insert_form_input_tag(GtkAction * action, gpointer user_data)
 						   main_v->props.xhtml == 1 ? cap("<INPUT />") : cap("<INPUT>"), NULL);
 }
 
+static void
+htmlbar_insert_form_button_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<BUTTON>"), cap("</BUTTON>"));
+}
+
+static void
+htmlbar_insert_form_select_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<SELECT>"), cap("</SELECT>"));
+}
+
+static void
+htmlbar_insert_form_datalist_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<DATALIST>"), cap("</DATALIST>"));
+}
+
+static void
+htmlbar_insert_form_optgroup_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<OPTGROUP>"), cap("</OPTGROUP>"));
+}
+
+static void
+htmlbar_insert_form_option_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<OPTION>"), cap("</OPTION>"));
+}
+
 static void
 htmlbar_insert_form_textarea_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<TEXTAREA>"), cap("</TEXTAREA>"));
 }
 
+static void
+htmlbar_insert_form_keygen_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document,
+						   main_v->props.xhtml == 1 ? cap("<KEYGEN />") : cap("<KEYGEN>"), NULL);
+}
+
+static void
+htmlbar_insert_form_output_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<OUTPUT>"), cap("</OUTPUT>"));
+}
+
+static void
+htmlbar_insert_form_progress_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<PROGRESS>"), cap("</PROGRESS>"));
+}
+
+static void
+htmlbar_insert_form_meter_tag(GtkAction * action, gpointer user_data)
+{
+	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<METER>"), cap("</METER>"));
+}
+
 static void
 htmlbar_insert_frame_tag(GtkAction * action, gpointer user_data)
 {
@@ -1222,13 +1296,13 @@ htmlbar_insert_middot(GtkAction * action, gpointer user_data)
 }
 
 void
-htmlbar_toolbar_show(Thtmlbarwin * hbw, gboolean show)
+htmlbar_toolbar_show(Thtmlbarwin * hbw, Thtmlbarsession *hbs, gboolean show)
 {
 	if (show) {
 		if (hbw->handlebox)
 			gtk_widget_show(hbw->handlebox);
 		else
-			htmlbar_toolbar_create(hbw);
+			htmlbar_toolbar_create(hbw, hbs);
 	} else {
 		if (hbw->handlebox)
 			gtk_widget_hide(hbw->handlebox);
@@ -1242,13 +1316,10 @@ htmlbar_toolbar_show_toogle(GtkAction * action, gpointer user_data)
 	Thtmlbarsession *hbs;
 
 	hbs = g_hash_table_lookup(htmlbar_v.lookup, hbw->bfwin->session);
-	if (hbs) {
-		hbs->view_htmlbar = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
-	} else {
-		DEBUG_MSG("htmlbar_toolbar_show_toogle, ERROR, no htmlbarsession ???\n");
-	}
-
-	htmlbar_toolbar_show(hbw, gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)));
+	if (!hbs) 
+		return;
+	hbs->view_htmlbar = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
+	htmlbar_toolbar_show(hbw, hbs, gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)));
 }
 
 static void
@@ -1332,7 +1403,6 @@ htmlbar_insert_section_tag(GtkAction * action, gpointer user_data)
 static void
 htmlbar_dialog_audio(GtkAction * action, gpointer user_data)
 {
-	
 	audio_dialog(BFWIN(user_data), NULL);
 }
 
@@ -1467,7 +1537,7 @@ static const GtkActionEntry htmlbar_actions[] = {
 	{"UppercaseETH", NULL, N_("_ETH (Icelandic) Ð"), NULL, NULL, G_CALLBACK(htmlbar_insert_eth)},
 	{"UppercaseTHORN", NULL, N_("_THORN (Icelandic) Þ"), NULL, NULL, G_CALLBACK(htmlbar_insert_thorn)},
 	{"LowercaseETH", NULL, N_("et_h (Icelandic) ð"), NULL, NULL, G_CALLBACK(htmlbar_insert_eth)},
-	{"LowercaseTHORN", NULL, N_("th_orn ÿ"), NULL, NULL, G_CALLBACK(htmlbar_insert_thorn)},
+	{"LowercaseTHORN", NULL, N_("th_orn þ"), NULL, NULL, G_CALLBACK(htmlbar_insert_thorn)},
 	{"NonLatinSharp", NULL, N_("Sharp _s ß"), NULL, NULL, G_CALLBACK(htmlbar_insert_sharp)},
 	{"NonLatinMicroSign", NULL, N_("_Micro sign µ"), NULL, NULL, G_CALLBACK(htmlbar_insert_micro_sign)},
 	{"SpecialOtherMenu", NULL, N_("_Other")},
@@ -1574,10 +1644,21 @@ static const GtkActionEntry htmlbar_actions[] = {
 	 G_CALLBACK(htmlbar_insert_definition_tag)},
 	{"ListListMenu", NULL, N_("Men_u"), NULL, N_("Menu"), G_CALLBACK(htmlbar_insert_menu_tag)},
 	{"FormMenu", NULL, N_("F_orm")},
-	{"FormForm", BF_STOCK_FORM, N_("_Form"), NULL, N_("Form"), G_CALLBACK(htmlbar_insert_form_tag)},
-	{"FormButton", NULL, N_("_Button"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_button_tag)},
+	{"FormForm", BF_STOCK_FORM, N_("F_orm"), NULL, N_("Form"), G_CALLBACK(htmlbar_insert_form_tag)},
+	{"FormFieldset", NULL, N_("_Fieldset"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_fieldset_tag)},
+	{"FormLegend", NULL, N_("_Legend"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_legend_tag)},
+	{"FormLabel", NULL, N_("L_abel"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_label_tag)},
 	{"FormInput", NULL, N_("_Input"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_input_tag)},
+	{"FormButton", NULL, N_("_Button"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_button_tag)},
+	{"FormSelect", NULL, N_("_Select"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_select_tag)},
+	{"FormDatalist", NULL, N_("_Datalist"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_datalist_tag)},
+	{"FormOptgroup", NULL, N_("Opt_group"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_optgroup_tag)},
+	{"FormOption", NULL, N_("Optio_n"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_option_tag)},
 	{"FormTextArea", NULL, N_("_Textarea"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_textarea_tag)},
+	{"FormKeygen", NULL, N_("_Keygen"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_keygen_tag)},
+	{"FormOutput", NULL, N_("O_utput"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_output_tag)},
+	{"FormProgress", NULL, N_("_Progress"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_progress_tag)},
+	{"FormMeter", NULL, N_("_Meter"), NULL, NULL, G_CALLBACK(htmlbar_insert_form_meter_tag)},
 	{"MiscMenu", NULL, N_("_Misc")},
 	{"MiscGenMETATag", NULL, N_("Insert Generator _META-Tag"), NULL, NULL,
 	 G_CALLBACK(htmlbar_insert_generator_meta_tag)},
@@ -1597,19 +1678,21 @@ static const GtkActionEntry htmlbar_actions[] = {
 	 G_CALLBACK(htmlbar_dialog_link_header)},
 	{"DialogsBody", BF_STOCK_BODY, N_("_Body..."), "<shift><alt>B", N_("Body..."),
 	 G_CALLBACK(htmlbar_dialog_body)},
-	{"DialogsAnchor", BF_STOCK_ANCHOR, N_("_Anchor..."), "<shift><alt>a", N_("Anchor / Hyperlink..."),
+	{"DialogsScript", BF_STOCK_SCRIPT, N_("_Script..."), "<shift><alt>P", N_("Script..."),
+	 G_CALLBACK(htmlbar_dialog_script)},
+	{"DialogsAnchor", BF_STOCK_ANCHOR, N_("_Anchor..."), "<shift><alt>A", N_("Anchor / Hyperlink..."),
 	 G_CALLBACK(htmlbar_dialog_anchor)},
-	{"DialogsEmail", BF_STOCK_EMAIL, N_("_E-mail..."), "<shift><alt>e", N_("E-mail..."),
+	{"DialogsEmail", BF_STOCK_EMAIL, N_("_E-mail..."), "<shift><alt>E", N_("E-mail..."),
 	 G_CALLBACK(htmlbar_dialog_email)},
-	{"DialogsRule", BF_STOCK_HRULE, N_("_Rule..."), "<shift><alt>r", N_("Horizontal Rule..."),
+	{"DialogsRule", BF_STOCK_HRULE, N_("_Rule..."), "<shift><alt>R", N_("Horizontal Rule..."),
 	 G_CALLBACK(htmlbar_dialog_rule)},
-	{"DialogsFont", BF_STOCK_FONT, N_("_Font..."), "<shift><alt>f", N_("Font..."),
+	{"DialogsFont", BF_STOCK_FONT, N_("_Font..."), "<shift><alt>F", N_("Font..."),
 	 G_CALLBACK(htmlbar_dialog_font)},
 	{"DialogsFontBase", BF_STOCK_FONT_BASE, N_("Basef_ont..."), NULL, N_("Base font..."),
 	 G_CALLBACK(htmlbar_dialog_basefont)},
 	{"DialogsQuicklist", BF_STOCK_LIST, N_("Quick_list..."), "<shift><alt>L", N_("Quick List..."),
 	 G_CALLBACK(htmlbar_dialog_quicklist)},
-	{"DialogsMeta", NULL, N_("_Meta..."), "<shift><alt>m", NULL, G_CALLBACK(htmlbar_dialog_meta)},
+	{"DialogsMeta", NULL, N_("_Meta..."), "<shift><alt>M", NULL, G_CALLBACK(htmlbar_dialog_meta)},
 	{"DialogsEmbed", NULL, N_("Embe_d..."), NULL, NULL, G_CALLBACK(htmlbar_dialog_embed)},
 	{"DialogsColor", NULL, N_("Select _Color..."), NULL, NULL, G_CALLBACK(htmlbar_dialog_color)},
 	{"DialogsTime", NULL, N_("Insert _Time..."), NULL, NULL, G_CALLBACK(htmlbar_dialog_time)},
@@ -1636,8 +1719,8 @@ static const GtkActionEntry htmlbar_actions[] = {
 	{"DialogsSpan", BF_STOCK_CSSSPAN, N_("S_pan..."), NULL, N_("Span..."), G_CALLBACK(htmlbar_dialog_span)},
 	{"DialogsDiv", BF_STOCK_CSSDIV, N_("_Div..."), "<shift><alt>D", N_("Div..."),
 	 G_CALLBACK(htmlbar_dialog_div)},
-	{"DialogsStyle", BF_STOCK_CSSSTYLE, N_("_Style..."), NULL, N_("Style..."),
-	 G_CALLBACK(htmlbar_dialog_style)},
+	{"MiscStyle", BF_STOCK_CSSSTYLE, N_("_Style"), NULL, N_("Style"),
+	 G_CALLBACK(htmlbar_misc_style)},
 	{"DialogsLinkStylesheet", BF_STOCK_LINK_STYLESHEET, N_("_Link to Stylesheet..."), NULL,
 	 N_("Link to Stylesheet..."),
 	 G_CALLBACK(htmlbar_dialog_style_link_to)},
@@ -1706,28 +1789,222 @@ htmlbar_menu_create(Thtmlbarwin * hbw)
 {
 	Tbfwin *bfwin = hbw->bfwin;
 	Thtmlbarsession *hbs;
-	GtkActionGroup *action_group;
 
 	static const GtkToggleActionEntry htmlbar_toggle_actions[] = {
 		{"ViewHTMLToolbar", NULL, N_("_HTML Toolbar"), NULL, NULL, G_CALLBACK(htmlbar_toolbar_show_toogle),
 		 TRUE},
 	};
 
-	action_group = gtk_action_group_new("htmlbarActions");
-	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE "_plugin_htmlbar");
-	gtk_action_group_add_actions(action_group, htmlbar_actions, G_N_ELEMENTS(htmlbar_actions), bfwin);
-	gtk_action_group_add_toggle_actions(action_group, htmlbar_toggle_actions,
+	hbw->actiongroup = gtk_action_group_new("htmlbarActions");
+	gtk_action_group_set_translation_domain(hbw->actiongroup, GETTEXT_PACKAGE "_plugin_htmlbar");
+	gtk_action_group_add_actions(hbw->actiongroup, htmlbar_actions, G_N_ELEMENTS(htmlbar_actions), bfwin);
+	gtk_action_group_add_toggle_actions(hbw->actiongroup, htmlbar_toggle_actions,
 										G_N_ELEMENTS(htmlbar_toggle_actions), hbw);
-	gtk_ui_manager_insert_action_group(bfwin->uimanager, action_group, 0);
-	g_object_unref(action_group);
+	gtk_ui_manager_insert_action_group(bfwin->uimanager, hbw->actiongroup, 0);
+	g_object_unref(hbw->actiongroup);
 	hbs = g_hash_table_lookup(htmlbar_v.lookup, bfwin->session);
 	if (hbs)
-		bfwin_set_menu_toggle_item(action_group, "ViewHTMLToolbar", hbs->view_htmlbar);
+		bfwin_set_menu_toggle_item(hbw->actiongroup, "ViewHTMLToolbar", hbs->view_htmlbar);
 
 }
 
+
+static GtkWidget *quickbar_create_popup_menu(gboolean add, const gchar *actionname);
+
+static gboolean
+quickbar_button_press_event_lcb(GtkWidget *widget,GdkEvent  *event,gpointer   user_data)
+{
+	if (event->button.button == 3) {
+		GtkWidget *p, *menu;
+		GtkAction *action;
+		p = gtk_widget_get_parent(widget);
+		if (!p)
+			return FALSE;
+		action = gtk_activatable_get_related_action(GTK_ACTIVATABLE(p));
+		if (!action)
+			return FALSE;
+		menu = quickbar_create_popup_menu(FALSE, gtk_action_get_name(action));
+		gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,3,event->button.time);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
+static void
+htmlbar_quickbar_add_item(Thtmlbarwin * hbw, const gchar *actionname)
+{
+	GtkWidget *toolitem;
+	GtkAction *action;
+	GList *children2, *tmplist2;
+	action = gtk_action_group_get_action(hbw->actiongroup, actionname);
+	if (!action)
+		return;
+	toolitem = gtk_action_create_tool_item(action);
+	if (!toolitem)
+		return;
+	children2 = gtk_container_get_children(GTK_CONTAINER(toolitem));
+	for (tmplist2=g_list_first(children2);tmplist2;tmplist2=g_list_next(tmplist2)) {
+		/*g_print("have child-child %p of type %s\n", tmplist2->data, G_OBJECT_TYPE_NAME(tmplist2->data));*/
+		g_signal_connect(tmplist2->data, "button-press-event", G_CALLBACK(quickbar_button_press_event_lcb), hbw);
+	}
+	g_list_free(children2);	
+	
+	gtk_toolbar_insert(GTK_TOOLBAR(hbw->quickbar_toolbar), GTK_TOOL_ITEM(toolitem), -1);	
+}
+
+static void
+add_to_quickbar_activate_lcb(GtkMenuItem *m, gpointer data)
+{
+	GList *tmplist;
+	gboolean firstitem;
+	/* first check if it is not a duplicate */
+	if (find_in_stringlist(htmlbar_v.quickbar_items, (gchar *)data)) {
+		DEBUG_MSG("add_to_quickbar_activate_lcb, duplicate entry for %s\n",(gchar *)data);
+		return;
+	}
+	firstitem = (htmlbar_v.quickbar_items==NULL);
+	
+	DEBUG_MSG("add_to_quickbar_activate_lcb, adding %s to quickbar_items\n", (gchar *)data);
+	htmlbar_v.quickbar_items = g_list_append(htmlbar_v.quickbar_items, g_strdup(data));
+	/* now loop over all the windows that have a htmlbar, and add the item */
+	for (tmplist=g_list_first(main_v->bfwinlist);tmplist;tmplist=g_list_next(tmplist)) {
+		Thtmlbarwin * hbw;
+		hbw = g_hash_table_lookup(htmlbar_v.lookup, tmplist->data);
+		if (hbw) {
+			if (firstitem) {
+				GList *children = gtk_container_get_children(GTK_CONTAINER(hbw->quickbar_toolbar));
+				if (children)
+					gtk_container_remove(GTK_CONTAINER(hbw->quickbar_toolbar), children->data);
+			}
+			
+			htmlbar_quickbar_add_item(hbw, (gchar *)data);
+		}
+	}
+}
+
+static void
+htmlbar_quickbar_remove_item(Thtmlbarwin * hbw, const gchar *actionname)
+{
+	GList *children, *tmplist;
+	children = gtk_container_get_children(GTK_CONTAINER(hbw->quickbar_toolbar));
+	for (tmplist=g_list_first(children);tmplist;tmplist=g_list_next(tmplist)) {
+/*		g_print("have child %p of type %s\n", tmplist->data, G_OBJECT_TYPE_NAME(tmplist->data));*/
+		GtkAction *action = gtk_activatable_get_related_action(tmplist->data);
+		if (action && strcmp(gtk_action_get_name(action), actionname)==0) {
+			gtk_container_remove(GTK_CONTAINER(hbw->quickbar_toolbar), tmplist->data);
+			return;
+		}
+		/*g_print("action=%p %s\n", action, action? gtk_action_get_name(action): "NULL");*/
+	}
+	g_list_free(children);
+}
+
+static void
+remove_from_quickbar_activate_lcb(GtkMenuItem *m, gpointer data)
+{
+	GList *tmplist;
+	DEBUG_MSG("remove_from_quickbar_activate_lcb, removing %s from quickbar_items\n", (gchar *)data);
+	htmlbar_v.quickbar_items = remove_from_stringlist(htmlbar_v.quickbar_items, data);
+	/* now loop over all the windows that have a htmlbar, and remove the item */
+	for (tmplist=g_list_first(main_v->bfwinlist);tmplist;tmplist=g_list_next(tmplist)) {
+		Thtmlbarwin * hbw;
+		hbw = g_hash_table_lookup(htmlbar_v.lookup, tmplist->data);
+		if (hbw) {
+			htmlbar_quickbar_remove_item(hbw, (const gchar *)data);
+		}
+	}
+}
+
+static GtkWidget *
+quickbar_create_popup_menu(gboolean add, const gchar *actionname)
+{
+	GtkWidget *menu, *menuitem;
+	menu = gtk_menu_new();
+	menuitem = gtk_menu_item_new_with_label(add ? _("Add to Quickbar") : _("Remove from Quickbar"));
+	g_signal_connect(G_OBJECT(menuitem), "activate", add ? G_CALLBACK(add_to_quickbar_activate_lcb) : G_CALLBACK(remove_from_quickbar_activate_lcb), (gpointer)actionname);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	gtk_widget_show_all(menu);
+	return menu;
+}
+
+static gboolean
+toolbar_button_press_event_lcb(GtkWidget *widget,GdkEvent  *event,gpointer   user_data)
+{
+	/*Thtmlbarwin * hbw=user_data;*/
+	DEBUG_MSG("toolbar_button_press_event_lcb, called for widget %p\n",widget);
+	if (event->button.button == 3) {
+		GtkWidget *p, *menu;
+		GtkAction *action;
+		p = gtk_widget_get_parent(widget);
+		if (!p)
+			return FALSE;
+		action = gtk_activatable_get_related_action(GTK_ACTIVATABLE(p));
+		if (!action)
+			return FALSE;
+		DEBUG_MSG("toolbar_button_press_event_lcb, add action %s \n", gtk_action_get_name(action));
+		
+		menu = quickbar_create_popup_menu(TRUE, gtk_action_get_name(action));
+		gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,3,event->button.time);
+/*		htmlbar_v.quickbar_items = g_list_prepend(htmlbar_v.quickbar_items, g_strdup(gtk_action_get_name(action)));*/
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+htmlbar_load_quickbar(Thtmlbarwin * hbw, GtkWidget *toolbar)
+{
+	GList *tmplist;
+	if (htmlbar_v.quickbar_items == NULL) {
+		GtkWidget *label;
+		GtkToolItem *ti = gtk_tool_item_new();
+		label = gtk_label_new(_("Right click any html toolbar button to add it to the Quickbar."));
+		/*gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);*/
+		gtk_container_add(GTK_CONTAINER(ti), label);
+		gtk_toolbar_insert(GTK_TOOLBAR(toolbar),ti,0);
+	}
+	for (tmplist=g_list_first(htmlbar_v.quickbar_items);tmplist;tmplist=g_list_next(tmplist)) {
+		htmlbar_quickbar_add_item(hbw, tmplist->data);
+	}
+}
+
+static void
+setup_items_for_quickbar(Thtmlbarwin * hbw, GtkWidget *toolbar)
+{
+	GList *children, *tmplist, *children2, *tmplist2;
+	DEBUG_MSG("setup_items_for_quickbar, about to connect signals to toolbar buttons\n");
+	children = gtk_container_get_children(GTK_CONTAINER(toolbar));
+	for (tmplist=g_list_first(children);tmplist;tmplist=g_list_next(tmplist)) {
+/*		g_print("have child %p of type %s\n", tmplist->data, G_OBJECT_TYPE_NAME(tmplist->data));*/
+		GtkAction *action = gtk_activatable_get_related_action(tmplist->data);
+		if (action) {
+			children2 = gtk_container_get_children(GTK_CONTAINER(tmplist->data));
+			for (tmplist2=g_list_first(children2);tmplist2;tmplist2=g_list_next(tmplist2)) {
+				/*g_print("have child-child %p of type %s\n", tmplist2->data, G_OBJECT_TYPE_NAME(tmplist2->data));*/
+				g_signal_connect(tmplist2->data, "button-press-event", G_CALLBACK(toolbar_button_press_event_lcb), hbw);
+			}
+			g_list_free(children2);			
+		}
+		/*g_print("action=%p %s\n", action, action? gtk_action_get_name(action): "NULL");*/
+	}
+	g_list_free(children);
+}
+
+static void
+notebook_switch_page_lcb(GtkNotebook *notebook,GtkWidget   *page,guint        page_num,gpointer     user_data)
+{
+	Thtmlbarwin * hbw = user_data;
+	Thtmlbarsession *hbs;
+	hbs = g_hash_table_lookup(htmlbar_v.lookup, hbw->bfwin->session);
+	if (hbs) {
+		DEBUG_MSG("htmlbar notebook_switch_page_lcb, set page %d\n", page_num);
+		hbs->notebooktab = page_num;
+	}
+}
+
 void
-htmlbar_toolbar_create(Thtmlbarwin * hbw)
+htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 {
 	Tbfwin *bfwin = hbw->bfwin;
 
@@ -1742,43 +2019,63 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw)
 	gtk_container_add(GTK_CONTAINER(hbw->handlebox), html_notebook);
 	gtk_box_pack_start(GTK_BOX(bfwin->toolbarbox), hbw->handlebox, FALSE, FALSE, 0);
 
+	hbw->quickbar_toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(hbw->quickbar_toolbar), GTK_TOOLBAR_ICONS);
+	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), hbw->quickbar_toolbar, gtk_label_new(_(" Quickbar ")));
+	htmlbar_load_quickbar(hbw, hbw->quickbar_toolbar);
+
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Standard ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFontsToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Fonts ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFormattingToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Formatting ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLTablesToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Tables ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFramesToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Frames ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFormsToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Forms ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLListToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" List ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLCSSToolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" CSS ")));
+	setup_items_for_quickbar(hbw, toolbar);
 
 	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLHTML5Toolbar");
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" HTML 5 ")));
-
+	setup_items_for_quickbar(hbw, toolbar);
+	
 	gtk_widget_show_all(hbw->handlebox);
+	
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(html_notebook), hbs->notebooktab);
+	DEBUG_MSG("htmlbar htmlbar_toolbar_create, make page %d active\n", hbs->notebooktab);
+	g_signal_connect(G_OBJECT(html_notebook), "switch-page", G_CALLBACK(notebook_switch_page_lcb), hbw);
+
+	
 }
 
 void htmlbar_load_ui(Thtmlbarwin * hbw)
@@ -1871,6 +2168,7 @@ htmlbar_register_stock_icons(void)
 			{pixmap_quick_start, BF_STOCK_QUICKSTART},
 			{pixmap_right, BF_STOCK_RIGHT},
 			{pixmap_samp, BF_STOCK_SAMP},
+			{pixmap_script, BF_STOCK_SCRIPT},
 			{pixmap_strikeout, BF_STOCK_STRIKEOUT},
 			{pixmap_table_capt, BF_STOCK_TABLE_CAPTION},
 			{pixmap_table_td, BF_STOCK_TABLE_TD},
