@@ -26,11 +26,11 @@
 
 void htmlbar_load_ui(Thtmlbarwin * hbw);
 void htmlbar_menu_create(Thtmlbarwin *hbw);
-void htmlbar_toolbar_create(Thtmlbarwin *hbw);
+void htmlbar_toolbar_create(Thtmlbarwin *hbw, Thtmlbarsession *hbs);
 
 void htmlbar_register_stock_icons(void);
 
-void htmlbar_toolbar_show(Thtmlbarwin *hbw, gboolean show);
+void htmlbar_toolbar_show(Thtmlbarwin *hbw, Thtmlbarsession *hbs, gboolean show);
 
 
 #endif /* HTMLBAR_UIMANAGER_H_ */
