@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * image.c - the thumbnail/multi-thumbnail dialogs
  *
- * Copyright (C) 2003-2010 Olivier Sessink
+ * Copyright (C) 2003-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,25 +14,27 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-/* indented with indent -ts4 -kr -l110   */
+/* indented with indent -psl -ts4 -kr -l110   */
 
-/* #define DEBUG */
+/*#define DEBUG */
 
 #include <string.h>
 
+#include "image.h"
+#include "cap.h"
 #include "htmlbar.h"
 #include "html_diag.h"
-#include "cap.h"
+#include "../bf_lib.h"
+#include "../dialog_utils.h"
 #include "../document.h"
 #include "../file.h"
 #include "../gtk_easy.h"
-#include "../bf_lib.h"
 #include "../stringlist.h"
 
-static GdkPixbufLoader *pbloader_from_filename(const gchar * filename)
+static GdkPixbufLoader *
+pbloader_from_filename(const gchar * filename)
 {
 	GdkPixbufLoader *pbloader;
 	GError *error = NULL;
@@ -55,7 +57,8 @@ static GdkPixbufLoader *pbloader_from_filename(const gchar * filename)
 	return pbloader;
 }
 
-static gchar *create_thumbnail_filename(gchar * filename)
+static gchar *
+create_thumbnail_filename(gchar * filename)
 {
 	gchar *retval, *tmp;
 	gint len = 0, size;
@@ -92,7 +95,8 @@ typedef struct {
 	guint adj_changed_id;
 } Timage_diag;
 
-void image_diag_destroy_cb(GtkWidget * widget, Timage_diag * imdg)
+void
+image_diag_destroy_cb(GtkWidget * widget, Timage_diag * imdg)
 {
 	html_diag_destroy_cb(widget, imdg->dg);
 	if (imdg->pb) {
@@ -104,15 +108,16 @@ void image_diag_destroy_cb(GtkWidget * widget, Timage_diag * imdg)
 	g_free(imdg);
 }
 
-static TcheckNsave_return async_thumbsave_lcb(TcheckNsave_status status, GError * gerror,
-											  gpointer callback_data)
+static TcheckNsave_return
+async_thumbsave_lcb(TcheckNsave_status status, GError * gerror, gpointer callback_data)
 {
 	DEBUG_MSG("async_thumbsave_lcb, status=%d\n", status);
 	/* TODO: handle error */
 	return CHECKNSAVE_CONT;
 }
 
-static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag * imdg)
+static void
+image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag * imdg)
 {
 	gchar *thestring, *finalstring;
 	gchar *thumbnailfilename, *filename;
@@ -187,22 +192,21 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag * imdg)
 
 		thestring =
 			insert_integer_if_spin(imdg->dg->spin[0], cap("WIDTH"), thestring,
-								   GTK_TOGGLE_BUTTON(imdg->dg->check[0])->active, 0);
+								   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(imdg->dg->check[0])), 0);
 		thestring =
 			insert_integer_if_spin(imdg->dg->spin[1], cap("HEIGHT"), thestring,
-								   GTK_TOGGLE_BUTTON(imdg->dg->check[1])->active, 0);
+								   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(imdg->dg->check[1])), 0);
 		if (!main_v->props.xhtml) {
 			thestring = insert_integer_if_spin(imdg->dg->spin[2], cap("BORDER"), thestring, FALSE, 1);
 		}
 		thestring = insert_integer_if_spin(imdg->dg->spin[3], cap("HSPACE"), thestring, FALSE, 0);
 		thestring = insert_integer_if_spin(imdg->dg->spin[4], cap("VSPACE"), thestring, FALSE, 0);
-		thestring = insert_string_if_entry(imdg->dg->entry[1], cap("NAME"), thestring, NULL);
-		thestring = insert_string_if_entry(imdg->dg->entry[2], cap("ALT"), thestring, "");
-		thestring = insert_string_if_entry(imdg->dg->entry[3], cap("USEMAP"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_ENTRY(imdg->dg->entry[1]), cap("NAME"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_ENTRY(imdg->dg->entry[2]), cap("ALT"), thestring, "");
+		thestring = insert_string_if_entry(GTK_ENTRY(imdg->dg->entry[3]), cap("USEMAP"), thestring, NULL);
 		thestring =
-			insert_string_if_entry(GTK_WIDGET(GTK_BIN(imdg->dg->combo[0])->child), cap("ALIGN"), thestring,
-								   NULL);
-		thestring = insert_string_if_entry(imdg->dg->entry[4], NULL, thestring, NULL);
+			insert_string_if_combobox(GTK_COMBO_BOX(imdg->dg->combo[0]), cap("ALIGN"), thestring, NULL);
+		thestring = insert_string_if_entry(GTK_ENTRY(imdg->dg->entry[4]), NULL, thestring, NULL);
 
 		finalstring = g_strconcat(thestring, (main_v->props.xhtml == 1) ? " />" : ">", NULL);
 		g_free(thestring);
@@ -218,18 +222,20 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag * imdg)
 	image_diag_destroy_cb(NULL, imdg);
 }
 
-void image_diag_cancel_clicked_cb(GtkWidget * widget, gpointer data)
+void
+image_diag_cancel_clicked_cb(GtkWidget * widget, gpointer data)
 {
 	image_diag_destroy_cb(NULL, data);
 }
 
-static void image_diag_finish(Timage_diag * imdg, GtkSignalFunc ok_func)
+static void
+image_diag_finish(Timage_diag * imdg, GCallback ok_func)
 {
 	GtkWidget *align, *hbox;
 
 	hbox = gtk_hbutton_box_new();
-	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 6);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_box_set_spacing(GTK_BOX(hbox), 6);
 
 	imdg->dg->obut = bf_stock_ok_button(ok_func, imdg);
 	imdg->dg->cbut = bf_stock_cancel_button(G_CALLBACK(image_diag_cancel_clicked_cb), imdg);
@@ -245,7 +251,8 @@ static void image_diag_finish(Timage_diag * imdg, GtkSignalFunc ok_func)
 	gtk_widget_show_all(GTK_WIDGET(imdg->dg->dialog));
 }
 
-static void image_dialog_set_pixbuf(Timage_diag * imdg)
+static void
+image_dialog_set_pixbuf(Timage_diag * imdg)
 {
 	gint pb_width, pd_height, toobig;
 	GdkPixbuf *tmp_pb;
@@ -270,8 +277,8 @@ static void image_dialog_set_pixbuf(Timage_diag * imdg)
 
 	tmp_pb =
 		gdk_pixbuf_scale_simple(imdg->pb, (pb_width / toobig), (pd_height / toobig),
-								main_v->globses.
-								image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST);
+								main_v->globses.image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR :
+								GDK_INTERP_NEAREST);
 
 	if (GTK_IS_WIDGET(imdg->im)) {
 		DEBUG_MSG("imdg->im == %p\n", imdg->im);
@@ -292,8 +299,9 @@ static void image_dialog_set_pixbuf(Timage_diag * imdg)
 	DEBUG_MSG("image_filename_changed() finished. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 }
 
-static void image_loaded_lcb(Topenfile_status status, GError * gerror, gchar * buffer, goffset buflen,
-							 gpointer callback_data)
+static void
+image_loaded_lcb(Topenfile_status status, GError * gerror, Trefcpointer * refp, goffset buflen,
+				 gpointer callback_data)
 {
 	Timage_diag *imdg = callback_data;
 	gboolean cleanup = TRUE;
@@ -314,7 +322,7 @@ static void image_loaded_lcb(Topenfile_status status, GError * gerror, gchar * b
 		break;
 	case OPENFILE_FINISHED:{
 			GError *error = NULL;
-			if (gdk_pixbuf_loader_write(imdg->pbloader, (const guchar *) buffer, buflen, &error)
+			if (gdk_pixbuf_loader_write(imdg->pbloader, (const guchar *) refp->data, buflen, &error)
 				&& gdk_pixbuf_loader_close(imdg->pbloader, &error)) {
 				imdg->pb = gdk_pixbuf_loader_get_pixbuf(imdg->pbloader);
 				if (imdg->pb) {
@@ -332,7 +340,8 @@ static void image_loaded_lcb(Topenfile_status status, GError * gerror, gchar * b
 	}
 }
 
-static void image_filename_changed(GtkWidget * widget, Timage_diag * imdg)
+static void
+image_filename_changed(GtkWidget * widget, Timage_diag * imdg)
 {
 	const gchar *filename;
 	gchar *tmp;
@@ -355,11 +364,11 @@ static void image_filename_changed(GtkWidget * widget, Timage_diag * imdg)
 	if ((tmp == NULL && filename[0] != '/') && imdg->dg->doc->uri) {
 		/* a relative path. create the absolute path. */
 		GFile *parent = g_file_get_parent(imdg->dg->doc->uri);
-		gchar *tmp; 
+		gchar *tmp;
 		/* filename is an URI, not a file path. the function g_file_resolve_relative_path
-		does not handle URI parts like %20 (a space) */
+		   does not handle URI parts like %20 (a space) */
 		tmp = g_uri_unescape_string(filename, NULL);
-		DEBUG_MSG("unescaped filename=%s\n",tmp);
+		DEBUG_MSG("unescaped filename=%s\n", tmp);
 		fullfilename = g_file_resolve_relative_path(parent, tmp);
 		g_free(tmp);
 		g_object_unref(parent);
@@ -372,7 +381,7 @@ static void image_filename_changed(GtkWidget * widget, Timage_diag * imdg)
 	if (fullfilename && g_file_query_exists(fullfilename, NULL)) {
 		gchar *name, *msg;
 		gchar *path = g_file_get_path(fullfilename);
-		DEBUG_MSG("path for fullfilename=%s\n",path);
+		DEBUG_MSG("path for fullfilename=%s\n", path);
 		imdg->pbloader = pbloader_from_filename(path);
 		g_free(path);
 
@@ -396,7 +405,8 @@ static void image_filename_changed(GtkWidget * widget, Timage_diag * imdg)
 	}
 }
 
-static void image_adjust_changed(GtkAdjustment * adj, Timage_diag * imdg)
+static void
+image_adjust_changed(GtkAdjustment * adj, Timage_diag * imdg)
 {
 	GdkPixbuf *tmp_pb;
 	gint tn_width, tn_height;
@@ -406,16 +416,17 @@ static void image_adjust_changed(GtkAdjustment * adj, Timage_diag * imdg)
 		return;
 	}
 
-	tn_width = imdg->adjustment->value * gdk_pixbuf_get_width(imdg->pb);
-	tn_height = imdg->adjustment->value * gdk_pixbuf_get_height(imdg->pb);
+	tn_width = gtk_adjustment_get_value(imdg->adjustment) * gdk_pixbuf_get_width(imdg->pb);
+	tn_height = gtk_adjustment_get_value(imdg->adjustment) * gdk_pixbuf_get_height(imdg->pb);
 
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[0]), tn_width);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[1]), tn_height);
 
+	/* TODO: move this to an idle callback so we will not block the UI */ 
 	tmp_pb =
 		gdk_pixbuf_scale_simple(imdg->pb, tn_width, tn_height,
-								main_v->globses.
-								image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST);
+								main_v->globses.image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR :
+								GDK_INTERP_NEAREST);
 
 	if (GTK_IS_WIDGET(imdg->im)) {
 		DEBUG_MSG("imdg->im == %p\n", imdg->im);
@@ -430,7 +441,8 @@ static void image_adjust_changed(GtkAdjustment * adj, Timage_diag * imdg)
 	DEBUG_MSG("image_adjust_changed finished. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 }
 
-void image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * data)
+void
+image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * data)
 {
 	static gchar *tagitems[] =
 		{ "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
@@ -456,58 +468,52 @@ void image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * d
 	imdg->adj_changed_id =
 		g_signal_connect(G_OBJECT(imdg->adjustment), "value_changed", G_CALLBACK(image_adjust_changed), imdg);
 	gtk_scale_set_digits(GTK_SCALE(scale), 3);
-	gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_DISCONTINUOUS);
 	gtk_box_pack_start(GTK_BOX(imdg->dg->vbox), scale, FALSE, FALSE, 0);
 
 	dgtable = html_diag_table_in_vbox(imdg->dg, 5, 9);
+
 	if (filename) {
-		imdg->dg->entry[0] = entry_with_text(filename, 1024);
+		imdg->dg->entry[0] = dialog_entry_in_table(filename, dgtable, 1, 7, 0, 1);
 	} else {
-		imdg->dg->entry[0] = entry_with_text(tagvalues[4], 1024);
+		imdg->dg->entry[0] = dialog_entry_in_table(tagvalues[4], dgtable, 1, 7, 0, 1);
 	}
-	bf_mnemonic_label_tad_with_alignment(_("_Image location:"), imdg->dg->entry[0], 0, 0.5, dgtable, 0, 1, 0,
-										 1);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[0], 1, 7, 0, 1);
+	dialog_mnemonic_label_in_table(_("_Image location:"), imdg->dg->entry[0], dgtable, 0, 1, 0, 1);
+
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), file_but_new(imdg->dg->entry[0], 0, bfwin), 7, 9, 0, 1);
 	g_signal_connect(G_OBJECT(imdg->dg->entry[0]), "changed", G_CALLBACK(image_filename_changed), imdg);
 
 	imdg->dg->spin[0] = spinbut_with_value(NULL, 0, 5000, 1.0, 10.0);
 	imdg->dg->check[0] = gtk_check_button_new_with_label("%");
 	parse_integer_for_dialog(tagvalues[0], imdg->dg->spin[0], NULL, imdg->dg->check[0]);
-	bf_mnemonic_label_tad_with_alignment(_("_Width:"), imdg->dg->spin[0], 0, 0.5, dgtable, 6, 7, 1, 2);
+	dialog_mnemonic_label_in_table(_("_Width:"), imdg->dg->spin[0], dgtable, 6, 7, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[0], 7, 8, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->check[0], 8, 9, 1, 2);
 
 	imdg->dg->spin[1] = spinbut_with_value(NULL, 0, 5000, 1.0, 10.0);
 	imdg->dg->check[1] = gtk_check_button_new_with_label("%");
 	parse_integer_for_dialog(tagvalues[1], imdg->dg->spin[1], NULL, imdg->dg->check[1]);
-	bf_mnemonic_label_tad_with_alignment(_("Hei_ght:"), imdg->dg->spin[1], 0, 0.5, dgtable, 6, 7, 2, 3);
+	dialog_mnemonic_label_in_table(_("Hei_ght:"), imdg->dg->spin[1], dgtable, 6, 7, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[1], 7, 8, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->check[1], 8, 9, 2, 3);
 
-	imdg->dg->entry[3] = entry_with_text(tagvalues[9], 1024);
-	bf_mnemonic_label_tad_with_alignment(_("_Usemap:"), imdg->dg->entry[3], 0, 0.5, dgtable, 0, 1, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[3], 1, 3, 1, 2);
+	imdg->dg->entry[3] = dialog_entry_in_table(tagvalues[9], dgtable, 1, 3, 1, 2);
+	dialog_mnemonic_label_in_table(_("_Usemap:"), imdg->dg->entry[3], dgtable, 0, 1, 1, 2);
 
-	imdg->dg->entry[1] = entry_with_text(tagvalues[8], 1024);
-	bf_mnemonic_label_tad_with_alignment(_("_Name:"), imdg->dg->entry[1], 0, 0.5, dgtable, 0, 1, 2, 3);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[1], 1, 3, 2, 3);
+	imdg->dg->entry[1] = dialog_entry_in_table(tagvalues[8], dgtable, 1, 3, 2, 3);
+	dialog_mnemonic_label_in_table(_("_Name:"), imdg->dg->entry[1], dgtable, 0, 1, 2, 3);
 
-	imdg->dg->entry[2] = entry_with_text(tagvalues[2], 1024);
-	bf_mnemonic_label_tad_with_alignment(_("Alternate _text:"), imdg->dg->entry[2], 0, 0.5, dgtable, 0, 1, 3,
-										 4);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[2], 1, 6, 3, 4);
+	imdg->dg->entry[2] = dialog_entry_in_table(tagvalues[2], dgtable, 1, 6, 3, 4);
+	dialog_mnemonic_label_in_table(_("Alternate _text:"), imdg->dg->entry[2], dgtable, 0, 1, 3, 4);
 
-	imdg->dg->entry[4] = entry_with_text(custom, 1024);
-	bf_mnemonic_label_tad_with_alignment(_("Custo_m:"), imdg->dg->entry[4], 0, 0.5, dgtable, 0, 1, 4, 5);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[4], 1, 6, 4, 5);
+	imdg->dg->entry[4] = dialog_entry_in_table(custom, dgtable, 1, 6, 4, 5);
+	dialog_mnemonic_label_in_table(_("Custo_m:"), imdg->dg->entry[4], dgtable, 0, 1, 4, 5);
 
 	imdg->dg->spin[3] = spinbut_with_value(tagvalues[5], 0, 500, 1.0, 5.0);
-	bf_mnemonic_label_tad_with_alignment(_("_Hspace:"), imdg->dg->spin[3], 0, 0.5, dgtable, 6, 7, 3, 4);
+	dialog_mnemonic_label_in_table(_("_Hspace:"), imdg->dg->spin[3], dgtable, 6, 7, 3, 4);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[3], 7, 9, 3, 4);
 
 	imdg->dg->spin[4] = spinbut_with_value(tagvalues[6], 0, 500, 1.0, 5.0);
-	bf_mnemonic_label_tad_with_alignment(_("_Vspace:"), imdg->dg->spin[4], 0, 0.5, dgtable, 6, 7, 4, 5);
+	dialog_mnemonic_label_in_table(_("_Vspace:"), imdg->dg->spin[4], dgtable, 6, 7, 4, 5);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[4], 7, 9, 4, 5);
 
 	popuplist = g_list_append(NULL, "bottom");
@@ -517,11 +523,11 @@ void image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * d
 	popuplist = g_list_append(popuplist, "right");
 	imdg->dg->combo[0] = combobox_with_popdown_sized(tagvalues[7], popuplist, 1, 90);
 	g_list_free(popuplist);
-	bf_mnemonic_label_tad_with_alignment(_("_Align:"), imdg->dg->combo[0], 0, 0.5, dgtable, 3, 4, 1, 2);
+	dialog_mnemonic_label_in_table(_("_Align:"), imdg->dg->combo[0], dgtable, 3, 4, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->combo[0], 4, 6, 1, 2);
 	if (!main_v->props.xhtml) {
 		imdg->dg->spin[2] = spinbut_with_value(tagvalues[3], 0, 500, 1.0, 5.0);
-		bf_mnemonic_label_tad_with_alignment(_("Borde_r:"), imdg->dg->spin[2], 0, 0.5, dgtable, 3, 4, 2, 3);
+		dialog_mnemonic_label_in_table(_("Borde_r:"), imdg->dg->spin[2], dgtable, 3, 4, 2, 3);
 		gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[2], 4, 6, 2, 3);
 	}
 	if (filename || tagvalues[4]) {
@@ -534,7 +540,8 @@ void image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * d
 		g_free(custom);
 }
 
-void thumbnail_insert_dialog(Tbfwin * bfwin)
+void
+thumbnail_insert_dialog(Tbfwin * bfwin)
 {
 	image_insert_dialog_backend(NULL, bfwin, NULL);
 }
@@ -575,7 +582,8 @@ typedef struct {
 	Tmuthudia *mtd;
 } Timage2thumb;
 
-static void mt_dialog_destroy(GtkWidget * wid, Tmuthudia * mtd)
+static void
+mt_dialog_destroy(GtkWidget * wid, Tmuthudia * mtd)
 {
 	/* check if we have some images still loading, all images that have 'created == TRUE'
 	   are ready */
@@ -598,7 +606,8 @@ static void mt_dialog_destroy(GtkWidget * wid, Tmuthudia * mtd)
 }
 
 /* needs both pixbufs to get the width !! */
-static void mt_fill_string(Timage2thumb * i2t, GdkPixbuf * image, GdkPixbuf * thumb)
+static void
+mt_fill_string(Timage2thumb * i2t, GdkPixbuf * image, GdkPixbuf * thumb)
 {
 	gint tw, th, ow, oh;
 	gchar *relthumb, *tmp, *relimage;
@@ -667,7 +676,8 @@ static void mt_fill_string(Timage2thumb * i2t, GdkPixbuf * image, GdkPixbuf * th
 	g_free(relthumb);
 }
 
-static Timage2thumb *mt_next(Timage2thumb * i2t)
+static Timage2thumb *
+mt_next(Timage2thumb * i2t)
 {
 	GList *tmplist;
 	tmplist = g_list_find(i2t->mtd->images, i2t);
@@ -675,7 +685,8 @@ static Timage2thumb *mt_next(Timage2thumb * i2t)
 	return (tmplist) ? tmplist->data : NULL;
 }
 
-static Timage2thumb *mt_prev(Timage2thumb * i2t)
+static Timage2thumb *
+mt_prev(Timage2thumb * i2t)
 {
 	GList *tmplist;
 	tmplist = g_list_find(i2t->mtd->images, i2t);
@@ -684,7 +695,8 @@ static Timage2thumb *mt_prev(Timage2thumb * i2t)
 }
 
 /* TRUE if already inserted or successfully inserted, FALSE if not yet ready */
-static gboolean mt_print_string(Timage2thumb * i2t)
+static gboolean
+mt_print_string(Timage2thumb * i2t)
 {
 	if (i2t->string == NULL && i2t->created == TRUE) {
 		/* already added the HTML string */
@@ -713,7 +725,8 @@ static gboolean mt_print_string(Timage2thumb * i2t)
 
 static void mt_start_load(Timage2thumb * i2t);
 
-static gboolean mt_start_next_load(Timage2thumb * i2t)
+static gboolean
+mt_start_next_load(Timage2thumb * i2t)
 {
 	GList *tmplist;
 	for (tmplist = g_list_first(i2t->mtd->images); tmplist; tmplist = g_list_next(tmplist)) {
@@ -726,8 +739,9 @@ static gboolean mt_start_next_load(Timage2thumb * i2t)
 	return FALSE;
 }
 
-static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * buffer, goffset buflen,
-							gpointer callback_data)
+static void
+mt_openfile_lcb(Topenfile_status status, GError * gerror, Trefcpointer *refp, goffset buflen,
+				gpointer callback_data)
 {
 	Timage2thumb *i2t = callback_data;
 	switch (status) {
@@ -737,9 +751,11 @@ static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * bu
 	case OPENFILE_ERROR_CANCELLED:{
 			/* TODO: should we warn the user ?? */
 #ifdef DEBUG
-			gchar *path = g_file_get_path(i2t->imagename);
-			DEBUG_MSG("mt_openfile_lcb, some error! status=%d for image %s\n", status, path);
-			g_free(path);
+			{
+				gchar *path = g_file_get_path(i2t->imagename);
+				DEBUG_MSG("mt_openfile_lcb, some error! status=%d for image %s\n", status, path);
+				g_free(path);
+			}
 #endif
 		}
 		break;
@@ -763,7 +779,7 @@ static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * bu
 			pbloader = pbloader_from_filename(path);
 			g_free(path);
 
-			if (gdk_pixbuf_loader_write(pbloader, (const guchar *) buffer, buflen, &error)
+			if (gdk_pixbuf_loader_write(pbloader, (const guchar *) refp->data, buflen, &error)
 				&& gdk_pixbuf_loader_close(pbloader, &error)) {
 				gint tw, th, ow, oh;
 				GdkPixbuf *image;
@@ -810,10 +826,10 @@ static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * bu
 					/*gdk_pixbuf_unref(image); will be unreffed with the loader! */
 					/* save the thumbnail */
 					if (strcmp(main_v->props.image_thumbnailtype, "jpeg") == 0) {
-						gdk_pixbuf_save_to_buffer(thumb, &buffer, &buflen, main_v->props.image_thumbnailtype,
+						gdk_pixbuf_save_to_buffer(thumb, (gchar **)&refp->data, &buflen, main_v->props.image_thumbnailtype,
 												  &error, "quality", "50", NULL);
 					} else {
-						gdk_pixbuf_save_to_buffer(thumb, &buffer, &buflen, main_v->props.image_thumbnailtype,
+						gdk_pixbuf_save_to_buffer(thumb, (gchar **)&refp->data, &buflen, main_v->props.image_thumbnailtype,
 												  &error, NULL);
 					}
 					g_object_unref(thumb);
@@ -823,8 +839,7 @@ static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * bu
 					} else {
 						GError *error = NULL;
 						GFileInfo *finfo;
-						Trefcpointer *refbuf = refcpointer_new(buffer);
-
+						refcpointer_ref(refp);
 						finfo = g_file_query_info(i2t->thumbname,
 												  BF_FILEINFO, G_FILE_QUERY_INFO_NONE, NULL, &error);
 						if (error != NULL) {
@@ -837,9 +852,9 @@ static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * bu
 						g_free(path);
 #endif
 						i2t->sf =
-							file_checkNsave_uri_async(i2t->thumbname, finfo, refbuf, buflen, FALSE, FALSE,
+							file_checkNsave_uri_async(i2t->thumbname, finfo, refp, buflen, FALSE, FALSE,
 													  async_thumbsave_lcb, NULL);
-						refcpointer_unref(refbuf);
+						refcpointer_unref(refp);
 					}
 				} else {
 					/* ok, this image is not valid, how do we continue ?? */
@@ -862,7 +877,8 @@ static void mt_openfile_lcb(Topenfile_status status, GError * gerror, gchar * bu
 	/* BUG: the last image that reaches this function should free 'mtd' after it is finished */
 }
 
-static void mt_start_load(Timage2thumb * i2t)
+static void
+mt_start_load(Timage2thumb * i2t)
 {
 #ifdef DEBUG
 	gchar *path = g_file_get_path(i2t->imagename);
@@ -872,7 +888,8 @@ static void mt_start_load(Timage2thumb * i2t)
 	i2t->of = file_openfile_uri_async(i2t->imagename, NULL, mt_openfile_lcb, i2t);
 }
 
-static Timage2thumb *mt_image2thumbnail(Tmuthudia * mtd, gchar * curi)
+static Timage2thumb *
+mt_image2thumbnail(Tmuthudia * mtd, gchar * curi)
 {
 	Timage2thumb *i2t;
 	gchar *tmp;
@@ -894,7 +911,8 @@ static Timage2thumb *mt_image2thumbnail(Tmuthudia * mtd, gchar * curi)
 	return i2t;
 }
 
-static void multi_thumbnail_ok_clicked(GtkWidget * widget, Tmuthudia * mtd)
+static void
+multi_thumbnail_ok_clicked(GtkWidget * widget, Tmuthudia * mtd)
 {
 	GSList *files = NULL, *tmplist;
 	GtkWidget *dialog;
@@ -902,13 +920,13 @@ static void multi_thumbnail_ok_clicked(GtkWidget * widget, Tmuthudia * mtd)
 
 	gtk_widget_hide(mtd->win);
 
-	if (GTK_TOGGLE_BUTTON(mtd->radio[0])->active) {
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[0]))) {
 		main_v->globses.image_thumbnailsizing_type = 0;
-	} else if (GTK_TOGGLE_BUTTON(mtd->radio[1])->active) {
+	} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[1]))) {
 		main_v->globses.image_thumbnailsizing_type = 1;
-	} else if (GTK_TOGGLE_BUTTON(mtd->radio[2])->active) {
+	} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[2]))) {
 		main_v->globses.image_thumbnailsizing_type = 2;
-	} else if (GTK_TOGGLE_BUTTON(mtd->radio[3])->active) {
+	} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[3]))) {
 		main_v->globses.image_thumbnailsizing_type = 3;
 	}
 	{
@@ -952,24 +970,26 @@ static void multi_thumbnail_ok_clicked(GtkWidget * widget, Tmuthudia * mtd)
 	/* BUG: should we free the list of files now ?? */
 }
 
-static void multi_thumbnail_cancel_clicked(GtkWidget * widget, Tmuthudia * mtd)
+static void
+multi_thumbnail_cancel_clicked(GtkWidget * widget, Tmuthudia * mtd)
 {
 	mt_dialog_destroy(NULL, mtd);
 }
 
-static void multi_thumbnail_radio_toggled_lcb(GtkToggleButton * togglebutton, Tmuthudia * mtd)
+static void
+multi_thumbnail_radio_toggled_lcb(GtkToggleButton * togglebutton, Tmuthudia * mtd)
 {
 	/* only call this for activate, not for de-activate */
-	if (togglebutton->active) {
-		if (GTK_TOGGLE_BUTTON(mtd->radio[0])->active) {
+	if (gtk_toggle_button_get_active(togglebutton)) {
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[0]))) {
 			gtk_widget_hide(mtd->spins[1]);
 			gtk_widget_hide(mtd->spinlabels[1]);
 			gtk_label_set_text(GTK_LABEL(mtd->spinlabels[0]), _("Scaling (%)"));
-		} else if (GTK_TOGGLE_BUTTON(mtd->radio[1])->active) {
+		} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[1]))) {
 			gtk_widget_hide(mtd->spins[1]);
 			gtk_widget_hide(mtd->spinlabels[1]);
 			gtk_label_set_text(GTK_LABEL(mtd->spinlabels[0]), _("Width"));
-		} else if (GTK_TOGGLE_BUTTON(mtd->radio[2])->active) {
+		} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mtd->radio[2]))) {
 			gtk_widget_hide(mtd->spins[1]);
 			gtk_widget_hide(mtd->spinlabels[1]);
 			gtk_label_set_text(GTK_LABEL(mtd->spinlabels[0]), _("Height"));
@@ -981,7 +1001,8 @@ static void multi_thumbnail_radio_toggled_lcb(GtkToggleButton * togglebutton, Tm
 	}
 }
 
-void multi_thumbnail_dialog(Tbfwin * bfwin)
+void
+multi_thumbnail_dialog(Tbfwin * bfwin)
 {
 	Tmuthudia *mtd;
 	GtkWidget *vbox, *hbox, *but, *table, *label, *scrolwin, *textview;
@@ -1046,8 +1067,8 @@ void multi_thumbnail_dialog(Tbfwin * bfwin)
 	gtk_box_pack_start(GTK_BOX(vbox), scrolwin, TRUE, TRUE, 0);
 
 	hbox = gtk_hbutton_box_new();
-	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_box_set_spacing(GTK_BOX(hbox), 1);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 	but = bf_stock_cancel_button(G_CALLBACK(multi_thumbnail_cancel_clicked), mtd);
 	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, FALSE, 5);
