@@ -429,6 +429,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.restore_dimensions, "restore_dimensions:", 1, TRUE);
 
 	init_prop_integer(&config_rc, &main_v->props.left_panel_left, "left_panel_left:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.save_accelmap, "save_accelmap:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 15, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.backup_file, "backup_file:", 1, TRUE);
@@ -851,32 +852,55 @@ rcfile_save_main(void)
 	return ret;
 }
 
-#ifndef WIN32
-#define DIR_MODE (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)	/* same as 0755 */
-#endif
 void
 rcfile_check_directory(void)
 {
-	gchar *rcdir = g_strconcat(g_get_home_dir(), "/." PACKAGE, NULL);
-	DEBUG_PATH("test if rcfile directory %s exists\n", rcdir);
-	if (!g_file_test(rcdir, G_FILE_TEST_IS_DIR)) {
-#ifndef WIN32
-		mkdir(rcdir, DIR_MODE);
-#else
-		mkdir(rcdir);
-#endif
+	GFile *file, *rcpath;
+	GError *gerror = NULL;
+
+	file = g_file_new_for_path(g_get_home_dir());
+	rcpath = g_file_get_child(file, ".bluefish");
+
+	DEBUG_PATH("test if rcfile directory %s exists\n", rcpath);
+	if (g_file_query_file_type(rcpath, G_FILE_QUERY_INFO_NONE, NULL) == G_FILE_TYPE_UNKNOWN) {
+		g_file_make_directory(rcpath, NULL, &gerror);
+		if (gerror) {
+			/* Do something with error */
+			g_error_free(gerror);
+		}
 	}
-	g_free(rcdir);
+
+	g_object_unref(rcpath);
+	g_object_unref(file);
+}
+
+static gchar *
+get_user_accelmap(void)
+{
+	return g_strconcat(g_get_home_dir(), "/." PACKAGE "/menudump_2", NULL);
 }
 
 void
 rcfile_save_accelerators(void)
 {
-	gchar *shortcutfilename = g_strconcat(g_get_home_dir(), "/." PACKAGE "/menudump_2", NULL);
+	gchar *shortcutfilename = get_user_accelmap();
 	gtk_accel_map_save(shortcutfilename);
 	g_free(shortcutfilename);
 }
 
+void
+rcfile_load_accelerators(gboolean defaultmap)
+{
+	gchar *shortcutfilename;
+	if (defaultmap) {
+		shortcutfilename = g_strdup(PKGDATADIR"/default_accelmap");
+	} else {
+		shortcutfilename = get_user_accelmap();
+	}
+	gtk_accel_map_load(shortcutfilename);
+	g_free(shortcutfilename);
+}
+
 static GHashTable *
 return_globalsession_configlist(gboolean init_values)
 {
@@ -933,12 +957,13 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_integer(&configlist, &session->autocomplete, "autocomplete:", 1, FALSE);
 	init_prop_integer(&configlist, &session->show_mbhl, "show_mbhl:", 1, FALSE);
 	
-	init_prop_integer(&configlist, &session->snr3_type, "snr_type:", 1, 0);
-	init_prop_integer(&configlist, &session->snr3_replacetype, "snr_replacetype:", 1, 0);
-	init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, 0);
-	init_prop_integer(&configlist, &session->snr3_casesens, "snr_casesens:", 1, 0);
-	init_prop_integer(&configlist, &session->snr3_escape_chars, "snr_escape_chars:", 1, 0);
-	init_prop_integer(&configlist, &session->snr3_dotmatchall, "snr_dotmatchall:", 1, 1);
+	init_prop_integer(&configlist, &session->snr3_type, "snr_type:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_replacetype, "snr_replacetype:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_casesens, "snr_casesens:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_escape_chars, "snr_escape_chars:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_dotmatchall, "snr_dotmatchall:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_recursion_level, "snr_recursion_level:", 5, FALSE);
 
 	init_prop_integer(&configlist, &session->display_right_margin, "display_right_margin:", 0, FALSE);
 #ifdef HAVE_LIBENCHANT
