@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snr3.c - search and replace
  *
- * Copyright (C) 2011 Olivier Sessink
+ * Copyright (C) 2011,2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,6 +70,8 @@ Tsnr3profiling s3profiling= {NULL};
 
 #endif
 
+static const gchar *searchresulttagname = "searchresult";
+
 typedef struct {
 	Tdocument *doc;
 	Tsnr3run *s3run;
@@ -134,7 +136,7 @@ remove_all_highlights_in_doc(Tdocument * doc)
 		return;
 		
 	tagtable = gtk_text_buffer_get_tag_table(doc->buffer);
-	tag = gtk_text_tag_table_lookup(tagtable, "snr3match");
+	tag = gtk_text_tag_table_lookup(tagtable, searchresulttagname);
 	if (!tag)
 		return;
 	
@@ -144,18 +146,16 @@ remove_all_highlights_in_doc(Tdocument * doc)
 
 static void highlight_result(Tsnr3result *s3result) {
 	GtkTextIter itstart, itend;
-	static const gchar *tagname = "snr3match";
 	static GtkTextTag *tag=NULL;
 	
 	gtk_text_buffer_get_iter_at_offset(DOCUMENT(s3result->doc)->buffer, &itstart, s3result->so);
 	gtk_text_buffer_get_iter_at_offset(DOCUMENT(s3result->doc)->buffer, &itend, s3result->eo);
 
 	if (!tag) {
-		tag = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(DOCUMENT(s3result->doc)->buffer), tagname);
+		tag = gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(DOCUMENT(s3result->doc)->buffer), searchresulttagname);
 		if (!tag) {
-			tag =
-				gtk_text_buffer_create_tag(DOCUMENT(s3result->doc)->buffer, tagname, "background", "#FFFF57", "foreground", "#000000",
-										   NULL);
+			g_warning("Failed to highlight search result, there is no text style 'searchresult' defined.");
+			return;
 		}
 	}
 	gtk_text_buffer_apply_tag(DOCUMENT(s3result->doc)->buffer, tag, &itstart, &itend);
@@ -184,8 +184,11 @@ typedef struct {
 
 static void 
 snr3run_update_offsets(Tsnr3run *s3run, Tdocument *doc, guint startpos, gint offset) {
+/* in the case of a replace of '/usr/bin/' in the string '/usr/bin/foo, /usr/bin/bar, /usr/bin/foobar'
+the startpos=9 (the end of the replaced result), offset=-9 and the first next result is at 14
+*/
 	GList *tmplist = g_list_first(s3run->results.head);
-	gint comparepos = (offset > 0) ? startpos : startpos - offset; 
+	gint comparepos = startpos/*(offset > 0) ? startpos : startpos - offset*/; 
 	DEBUG_MSG("snr3run_update_offsets, startpos=%d, offset=%d, comparepos=%d\n",startpos,offset,comparepos);
 	while (tmplist) {
 		if (doc == ((Tsnr3result *)tmplist->data)->doc 
@@ -252,7 +255,18 @@ s3result_replace(Tsnr3run *s3run, Tsnr3result *s3result, GMatchInfo *matchinfo)
 {
 	Toffsetupdate offsetupdate = {NULL,0,0};
 	s3run->in_replace=TRUE;
-	if (s3run->replacetype == snr3replace_string) {
+	if (s3run->type == snr3type_pcre && s3run->replacetype != snr3replace_string) {
+		gchar *tmp1, *tmp2;
+		DEBUG_MSG("s3result_replace, replace uppercase/lowercase\n");
+		tmp1 = doc_get_chars(s3result->doc, s3result->so, s3result->eo);
+		if (s3run->replacetype == snr3replace_upper) 
+			tmp2 = g_utf8_strup(tmp1, -1);
+		else
+			tmp2 = g_utf8_strdown(tmp1, -1);
+		doc_replace_text_backend(s3result->doc, tmp2, s3result->so, s3result->eo);
+		g_free(tmp1);
+		g_free(tmp2);
+	} else if (s3run->replacetype == snr3replace_string) {
 		if (s3run->type == snr3type_string) {
 			DEBUG_MSG("s3result_replace, replace %d:%d with %s\n", s3result->so, s3result->eo, s3run->replacereal);
 			doc_replace_text_backend(s3result->doc, s3run->replacereal, s3result->so, s3result->eo);
@@ -267,16 +281,6 @@ s3result_replace(Tsnr3run *s3run, Tsnr3result *s3result, GMatchInfo *matchinfo)
 				g_free(newstr);
 			}
 		}
-	} else {
-		gchar *tmp1, *tmp2;
-		tmp1 = doc_get_chars(s3result->doc, s3result->so, s3result->eo);
-		if (s3run->replacetype == snr3replace_upper) 
-			tmp2 = g_utf8_strup(tmp1, -1);
-		else
-			tmp2 = g_utf8_strdown(tmp1, -1);
-		doc_replace_text_backend(s3result->doc, tmp2, s3result->so, s3result->eo);
-		g_free(tmp1);
-		g_free(tmp2);
 	}
 	offsetupdate.startingpoint = s3result->eo;
 	offsetupdate.doc = s3result->doc;
@@ -305,6 +309,7 @@ s3run_replace_current(Tsnr3run *s3run)
 	if (!current)
 		return;
 	s3result = current->data;
+	doc_unre_new_group(s3result->doc);
 	offsetupdate = s3result_replace(s3run, s3result, NULL);
 
 	snr3result_free(s3result, s3run);
@@ -312,13 +317,14 @@ s3run_replace_current(Tsnr3run *s3run)
 	g_queue_delete_link(&s3run->results, current);
 	s3run->current = next;
 	
-	if (s3run->current) {
-		scroll_to_result(s3run->current->data, GTK_WINDOW(((TSNRWin *)s3run->dialog)->dialog));
-	}
 	if (offsetupdate.offset != 0) {
 		/* now re-calculate all the offsets in the results lists!!!!!!!!!!! */
 		snr3run_update_offsets(s3run, offsetupdate.doc, offsetupdate.startingpoint, offsetupdate.offset);
 	}
+	if (s3run->current) {
+		scroll_to_result(s3run->current->data, GTK_WINDOW(((TSNRWin *)s3run->dialog)->dialog));
+	}
+
 }
 
 static Tsnr3result * sn3run_add_result(Tsnr3run *s3run, gulong so, gulong eo, gpointer doc) {
@@ -546,6 +552,9 @@ void
 snr3_run_in_doc(Tsnr3run *s3run, Tdocument *doc, gint so, gint eo, gboolean update) {
 	Truninidle *rii;
 	
+	if (!s3run->queryreal && !s3run->regex)
+		return; 
+	
 	rii = g_slice_new(Truninidle);
 	rii->doc = doc;
 	rii->s3run=s3run;
@@ -682,11 +691,9 @@ snr3_run(Tsnr3run *s3run, TSNRWin *snrwin, Tdocument *doc, void (*callback)(void
 			if (s3run->replaceall) {
 				DEBUG_MSG("scope files, run with filepattern %s\n", s3run->filepattern);
 				s3run->showinoutputbox=TRUE;
-				s3run->recursive = TRUE;
 				snr3_run_in_files(s3run);
 			} else if (s3run->findall) { 
 				s3run->showinoutputbox=TRUE;
-				s3run->recursive = TRUE;
 				snr3_run_in_files(s3run);
 			}
 		break;
@@ -802,6 +809,13 @@ static gint update_snr3run(Tsnr3run *s3run) {
 		g_free(s3run->replacereal);
 		s3run->replacereal=NULL;
 	}
+	if (!s3run->query || s3run->query[0] == '\0') {
+		DEBUG_MSG("update_snr3run, empty query, return -1\n");
+		gtk_label_set_markup(GTK_LABEL(((TSNRWin *)s3run->dialog)->searchfeedback), "<span foreground=\"red\">Empty search string</span>");
+		gtk_widget_show(((TSNRWin *)s3run->dialog)->searchfeedback);
+		return -1;
+	}
+	
 	if (s3run->type == snr3type_pcre) {
 		if (!compile_regex(s3run)) {
 			g_free(s3run->query);
@@ -904,9 +918,17 @@ handle_changed_in_snr3doc(Tsnr3run *s3run, Tdocument *doc, gint pos, gint len) {
 	Toffsetupdate offsetupdate = {doc,pos,len};
 	Truninidle *rii;
 	gint comparepos;
-	if (s3run->in_replace) {
+	if (s3run->in_replace || s3run->scope == snr3scope_files) {
 		return;
 	}
+	if (s3run->scope != snr3scope_alldocs) { /* allfiles does already return in the 
+			previous check, so if it is not alldocs it must be in a single document */
+		GList *tmplist = g_list_first(s3run->results.head);
+		/* see if the user was actually searching in this doc */
+		if (tmplist && ((Tsnr3result *)tmplist->data)->doc != doc)
+			return;
+	}
+	
 	comparepos = (len > 0) ? pos : pos - len;
 	if (s3run->so > comparepos) {
 		s3run->so += len;
@@ -961,13 +983,18 @@ static void
 snr3_docdestroy_cb(Tdocument *doc, gpointer data)
 {
 	Tsnr3run *s3run = data;
+	GList *tmplist;
 	/* see if this is the current document of an ongoing search, if so, cancel the search */
 	DEBUG_MSG("snr3_docdestroy_cb, doc=%p, s3run=%p\n",doc,s3run);
 	if (s3run->curdoc == doc || s3run->scope == snr3scope_alldocs) {
 		snr3_cancel_run(s3run);
 	}
 	/* remove any existing search results for this doc */
-	GList *tmplist=g_list_first(s3run->results.head);
+	if (s3run->current && ((Tsnr3result *)s3run->current)->doc == doc) {
+		s3run->current = NULL;
+	}	
+	
+	tmplist=g_list_first(s3run->results.head);
 	while(tmplist) {
 		GList *next = tmplist->next;
 		Tsnr3result *s3result=tmplist->data;
@@ -996,9 +1023,8 @@ snr3run_new(Tbfwin *bfwin, gpointer dialog)
 } 
 
 void snr3run_unrun(Tsnr3run *s3run) {
-	DEBUG_MSG("snr3run_unrun, runcont=%d\n",s3run->runcount);
+	DEBUG_MSG("snr3run_unrun, before decrement runcount=%d\n",s3run->runcount);
 	if (g_atomic_int_dec_and_test(&s3run->runcount)) {
-		DEBUG_MSG("runcount 0, run the callback\n");
 		s3run->callback(s3run);
 		DEBUG_MSG("runcount 0, after the callback\n");
 	}
@@ -1090,7 +1116,7 @@ static gint
 snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 {
 	const gchar *query, *replace;
-	gint type, replacetype, scope, dotmatchall, escapechars;
+	gint type, replacetype, scope, dotmatchall, escapechars, recursion_level;
 	gboolean is_case_sens;
 	gint retval=0;
 	GFile *basedir;
@@ -1107,6 +1133,7 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 	is_case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->matchCase));
 	dotmatchall = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->dotmatchall));
 	escapechars = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->escapeChars));
+	recursion_level = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(snrwin->recursion_level));
 	
 	basedir = g_file_new_for_commandline_arg(gtk_entry_get_text(GTK_ENTRY(snrwin->basedir)));
 	filepattern = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(snrwin->filepattern))));
@@ -1168,9 +1195,14 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 		DEBUG_MSG("filepattern =%s\n",filepattern);
 		retval |= 1;
 	}
+	if (s3run->recursion_level != recursion_level) {
+		snr3_cancel_run(s3run);
+		s3run->recursion_level = recursion_level;
+		retval |= 1;
+	}
 	if (!s3run->basedir || !g_file_equal(s3run->basedir, basedir)) {
 		snr3_cancel_run(s3run);
-		if (!s3run->basedir)
+		if (s3run->basedir)
 			g_object_unref(s3run->basedir);
 		s3run->basedir = basedir;
 		retval |= 1;
@@ -1195,6 +1227,7 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 		snrwin->bfwin->session->snr3_casesens = is_case_sens;
 		snrwin->bfwin->session->snr3_escape_chars = escapechars;
 		snrwin->bfwin->session->snr3_dotmatchall = dotmatchall;
+		snrwin->bfwin->session->snr3_recursion_level = recursion_level;
 	}
 	
 	DEBUG_MSG("snr3run_init_from_gui, return %d\n", retval);
@@ -1323,6 +1356,8 @@ static void snr_dialog_show_widgets(TSNRWin * snrwin) {
 
 	widget_set_visible(snrwin->filepattern, (scope == snr3scope_files));
 	widget_set_visible(snrwin->filepatternL, (scope == snr3scope_files));
+	widget_set_visible(snrwin->recursion_level, (scope == snr3scope_files));
+	widget_set_visible(snrwin->recursion_levelL, (scope == snr3scope_files));
 	widget_set_visible(snrwin->basedir, (scope == snr3scope_files));
 	widget_set_visible(snrwin->basedirL, (scope == snr3scope_files));
 	widget_set_visible(snrwin->basedirB, (scope == snr3scope_files));
@@ -1536,7 +1571,13 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 	g_object_unref(lstore);*/
 	snrwin->filepattern = combobox_with_popdown(NULL, bfwin->session->filegloblist, TRUE);
 	snrwin->filepatternL = dialog_mnemonic_label_in_table(_("Filename pattern: "), snrwin->filepattern, table, 0, 1, currentrow, currentrow+1);
-	gtk_table_attach(GTK_TABLE(table), snrwin->filepattern, 1, 4, currentrow, currentrow+1,GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), snrwin->filepattern, 1, 2, currentrow, currentrow+1,GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
+
+	snrwin->recursion_level = gtk_spin_button_new_with_range(0, 1000, 1);
+	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(snrwin->recursion_level), 0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(snrwin->recursion_level), bfwin->session->snr3_recursion_level);
+	snrwin->recursion_levelL = dialog_mnemonic_label_in_table(_("Recursion level: "), snrwin->recursion_level, table, 2, 3, currentrow, currentrow+1);
+	gtk_table_attach(GTK_TABLE(table), snrwin->recursion_level, 3, 4, currentrow, currentrow+1,GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 	
 	currentrow++;
 	
@@ -1604,9 +1645,9 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 	snrwin->backButton = gtk_dialog_add_button(GTK_DIALOG(snrwin->dialog), GTK_STOCK_GO_BACK, SNR_RESPONSE_BACK);
 	snrwin->findButton = gtk_dialog_add_button(GTK_DIALOG(snrwin->dialog), GTK_STOCK_GO_FORWARD, SNR_RESPONSE_FIND);
 	
-	snrwin->findAllButton = gtk_dialog_add_button(GTK_DIALOG(snrwin->dialog), _("Find All"), SNR_RESPONSE_FIND_ALL);
+	snrwin->findAllButton = gtk_dialog_add_button(GTK_DIALOG(snrwin->dialog), _("_Find All"), SNR_RESPONSE_FIND_ALL);
 	
-	snrwin->bookmarkButton = gtk_dialog_add_button(GTK_DIALOG(snrwin->dialog), _("_Bookmark All"), SNR_RESPONSE_BOOKMARK_ALL);
+	snrwin->bookmarkButton = gtk_dialog_add_button(GTK_DIALOG(snrwin->dialog), _("B_ookmark All"), SNR_RESPONSE_BOOKMARK_ALL);
 	/*gtk_dialog_set_response_sensitive(GTK_DIALOG(snrwin->dialog), SNR_RESPONSE_FIND, FALSE); */
 	/*snr_comboboxentry_changed(GTK_COMBO_BOX_ENTRY(snrwin->search), snrwin);*/
 	gtk_widget_show_all(GTK_WIDGET(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(snrwin->dialog)))));
